package mcsimul;

import java.awt.*;
import java.awt.event.*;
import java.util.*;


/**
 * This file is part of MCSimul.
 * MCSimul is a Java application/applet, which does 2-D atomic Monte-Carlo simulation,
 * together with a graphical user interface
 * Copyright 2008, Michael Leitner
 *
 * As stated below, you are free to do whatever you want with this program, as long as
 * it conforms to the licence, but I would be glad to hear if someone found it useful,
 * so please tell me if so under michael.leitner@univie.ac.at
 *
 * MCSimul is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MCSimul is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

public class MCFrame extends Frame implements Runnable, ActionListener {
    int dimx = 50, dimy = 50,attemptFrequency=10;
    double anteil = .1;
    Configuration conf = new Configuration(dimx, dimy, anteil);
    MCCanvas mccanvas=new MCCanvas(conf);
    ControlPanel controlPanel=new ControlPanel(this,conf);
    java.text.DecimalFormat df;
    
    public static void main(String[] args) {
        MCFrame grafik=new MCFrame();
    }
    
    public MCFrame() {
        super("Monte-Carlo Simulation");
        addWindowListener(new WindowClosingAdapter(true));
        java.text.DecimalFormatSymbols dummy =new java.text.DecimalFormatSymbols();
        dummy.setDecimalSeparator('.');
        df = new java.text.DecimalFormat("0.0##",dummy);  
        setBackground(Color.lightGray);
        setLayout(new FlowLayout());
        add(mccanvas);                        
        add(controlPanel);
        controlPanel.displayAF.setText(""+attemptFrequency);
        pack();
        setVisible(true);
        startAnimation();
    }

    public void actionPerformed(ActionEvent event) {
        Object obj = event.getSource();
        if (obj instanceof Button) {
            String cmd = event.getActionCommand();
            if (cmd.equals("paint now!")) {
                try {
                    conf.paintAtoms(controlPanel.getPaintAtoms());
                    mccanvas.repaint();
                } catch (NumberFormatException e) {}
                controlPanel.rate.setText(" " + String.valueOf(df.format(conf.getRate())));
            } else {                    
                try {
                    conf.setTemp(controlPanel.getTemp());
                    conf.setU(controlPanel.getU());
                    conf.setJ(controlPanel.getJ());
                } catch (NumberFormatException e) {}           
            } 
        } else 
            if (obj.equals(controlPanel.displayAF)) {
                try {
                    attemptFrequency=controlPanel.getAttemptFrequency();
                } catch (NumberFormatException e) {}
            } 

    }

    public void startAnimation() {
        Thread th = new Thread(this);
        th.start();
    }

    public void run() {
        int i;
        while (true) {
            if (attemptFrequency < 1) {
                try {
                    Thread.sleep(100);
                } catch (InterruptedException e) {
                }
            }
            else if (attemptFrequency < 20) {
                conf.step();
                mccanvas.repaintMod();
                try {
                    Thread.sleep((int) (1000. / attemptFrequency));
                } catch (InterruptedException e) {
                }
            }
            else {
                for (i=0;i<attemptFrequency/10.;i++) conf.step();
                mccanvas.repaint();
                try {
                    Thread.sleep(100);
                } catch (InterruptedException e) {
                }
            }
        }
    }
}

class ControlPanel extends Panel {
    Label rate;
    private TextField displayTemp, displayJ, displayU, paintAtoms;
    public TextField displayAF;
    
    public ControlPanel(ActionListener al,Configuration conf) {
        java.text.DecimalFormatSymbols dummy =new java.text.DecimalFormatSymbols();
        dummy.setDecimalSeparator('.');
        java.text.DecimalFormat df = new java.text.DecimalFormat("0.0##",dummy);          
        
        GridBagLayout gridbag = new GridBagLayout();
        setLayout(gridbag);
        GridBagConstraints cl = new GridBagConstraints();
        GridBagConstraints cr = new GridBagConstraints();
        cl.anchor=GridBagConstraints.WEST;
        cl.fill = GridBagConstraints.HORIZONTAL;  
        cl.gridwidth = 1; 
        cr.anchor=GridBagConstraints.WEST;
        cr.fill = GridBagConstraints.HORIZONTAL;  
        cr.gridwidth = GridBagConstraints.REMAINDER; 
        
        Label label6 = new Label("rate of blue:");
        gridbag.setConstraints(label6, cl);
        add(label6);
        rate = new Label(" "+String.valueOf(df.format(conf.getRate())));
        gridbag.setConstraints(rate, cr);
        add(rate);
        
        Label label1=new Label("paint atoms:");
        gridbag.setConstraints(label1, cl);
        add(label1);
        paintAtoms = new TextField("0", 5);
        gridbag.setConstraints(paintAtoms, cr);
        add(paintAtoms);
        
        Button paintNow=new Button("paint now!");
        paintNow.addActionListener(al);
        gridbag.setConstraints(paintNow, cr);
        add(paintNow);
        
        Label label2=new Label("T:");
        gridbag.setConstraints(label2, cl);
        add(label2);
        displayTemp = new TextField(String.valueOf(df.format(conf.getTemp())), 5);
        gridbag.setConstraints(displayTemp, cr);
        add(displayTemp);
        
        Label label3=new Label("J:");
        gridbag.setConstraints(label3, cl);
        add(label3);
        displayJ = new TextField(String.valueOf(df.format(conf.getJ())), 5);
        gridbag.setConstraints(displayJ, cr);
        add(displayJ);
        
        Label label4=new Label("U:");
        gridbag.setConstraints(label4, cl);
        add(label4);
        displayU = new TextField(String.valueOf(df.format(conf.getU())), 5);
        gridbag.setConstraints(displayU, cr);
        add(displayU);
        
        Button updateParam=new Button("update parameters");
        updateParam.addActionListener(al);
        gridbag.setConstraints(updateParam, cr);
        add(updateParam);
             
        Label label5=new Label("Attempts/sec:");
        gridbag.setConstraints(label5, cl);
        add(label5);
        displayAF = new TextField(7);
        gridbag.setConstraints(displayAF, cr);
        displayAF.addActionListener(al);
        add(displayAF);
    }    
    
    public int getPaintAtoms() {
        return Integer.parseInt(paintAtoms.getText());
    }
    
    public double getTemp() {
        return Double.parseDouble(displayTemp.getText());
    }
    
    public double getU() {
        return Double.parseDouble(displayU.getText());
    }
    
    public double getJ() {
        return Double.parseDouble(displayJ.getText());
    }
    
    public int getAttemptFrequency() {
        return Integer.parseInt(displayAF.getText());
    }    
}


class Configuration {

    public int dimx, dimy,vacancyx,vacancyy,newx,newy;
    private int blue;
    Random r=new Random();
    public byte site[][];
    private double T=10.,U=0.,J=1.;    

    public Configuration(int dimx, int dimy, double ratio) {
        int i, j, dumx, dumy;
        this.dimx = dimx;
        this.dimy = dimy;
        site = new byte[dimx][dimy];
        for (i = 0; i < dimx; i++) {
            for (j = 0; j < dimy; j++) {
                site[i][j] = 0;
            }
        }
        i = 0;
        while (i < ratio * (dimx * dimy-1)) {
            dumx = (int) (r.nextDouble() * dimx);
            dumy = (int) (r.nextDouble() * dimy);
            if (site[dumx][dumy] == 0) {
                site[dumx][dumy] = 1;
                i++;
            }
        }
        blue=i;
        i = 0;
        while (i < 1) {
            dumx = (int) (r.nextDouble() * dimx);
            dumy = (int) (r.nextDouble() * dimy);
            if (site[dumx][dumy] == 0) {
                vacancyx=dumx;
                vacancyy=dumy;
                i++;
            }
        }
    }
    
    public void paintAtoms(int number) {
        int i;
        if (number>0)   
            for (i=0;i<number;i++)
                blueplus();
        else
            for (i=0;i>number;i--)
                pinkplus();
    }
    
    private void pinkplus() {
        int i=0,dumx,dumy;
        if (blue>0) while (i < 1) {
            dumx = (int) (r.nextDouble() * dimx);
            dumy = (int) (r.nextDouble() * dimy);
            if (site[dumx][dumy] == 1) {
                site[dumx][dumy]=0;
                blue--;
                i++;
            }
        }
    }
    
    private void blueplus() {
        int i=0,dumx,dumy;
        if (blue<dimx*dimy-1) while (i < 1) {
            dumx = (int) (r.nextDouble() * dimx);
            dumy = (int) (r.nextDouble() * dimy);
            if ((site[dumx][dumy] == 0)&&(vacancyx!=dumx)&&(vacancyy!=dumy)) {
                site[dumx][dumy]=1;
                blue++;
                i++;
            }
        }
    }
    
    public void setTemp(double Tempnew) {
        T=Tempnew;
    }
    
    public double getTemp() {
        return T;
    }
    
    public void setU(double Unew) {
        U=Unew;
    }
    
    public double getU() {
        return U;
    }    
    
    public void setJ(double Jnew) {
        J=Jnew;
    }
    
    public double getJ() {
        return J;
    }
    
    public void step(){
        int dx,dy,dumint;
        double dum;
        dumint=r.nextInt(4);
        if (dumint==0) {
            dx=0;
            dy=(1+vacancyy)%dimy-vacancyy;
        } else if (dumint==1) {
            dx=0;
            dy=(-1+vacancyy+dimy)%dimy-vacancyy;
        } else if (dumint==2) {
            dy=0;
            dx=(1+vacancyx)%dimx-vacancyx;
        } else {
            dy=0;
            dx=(-1+vacancyx+dimx)%dimx-vacancyx;
        }
        dum=site[(vacancyx+1)%dimx][vacancyy]+site[(vacancyx-1+dimx)%dimx][vacancyy]+site[vacancyx][(vacancyy+1)%dimy]+site[vacancyx][(vacancyy-1+dimy)%dimy];
        dum=dum-site[vacancyx+dx][vacancyy+dy];
        dum=dum-site[(vacancyx+1+dx)%dimx][vacancyy+dy]-site[(vacancyx-1+dimx+dx)%dimx][vacancyy+dy]-site[vacancyx+dx][(vacancyy+1+dy)%dimy]-site[vacancyx+dx][(vacancyy-1+dimy+dy)%dimy];
        if (r.nextDouble()>1/(1+Math.exp((U+J*(2*site[vacancyx+dx][vacancyy+dy]-1))*dum/T))){
            site[vacancyx][vacancyy]=site[vacancyx+dx][vacancyy+dy];
            newx=vacancyx;
            newy=vacancyy;
            vacancyx=vacancyx+dx;
            vacancyy=vacancyy+dy;
            site[vacancyx][vacancyy]=0;
            
        }
    }   
    
    public double getRate(){
        return (double)blue/(double)(dimx*dimy-1);
    }
}

class MCCanvas extends Canvas {

    Configuration conf;
    Color pink, blue;
    byte paintMode=0;

    public MCCanvas(Configuration conf) {
        setSize(300, 300);
        setBackground(Color.black);
        this.conf=conf;
        init();
    }

    public void init() {
        pink = new Color(255, 0, 160);
        blue = new Color(25, 100, 200);
    }    
    
    public void repaintMod() {
        paintMode=1;
        repaint();
        paintMode=0;
    }
    
    @Override
    public void paint(Graphics g) {
        int width = this.getSize().width;
        int height = this.getSize().height;    
        double dumx = (double) width / conf.dimx;
        double dumy = (double) height / conf.dimy;
        if (paintMode == 0) {
            int i, j;
            for (i = 0; i < conf.dimx; i++) {
                for (j = 0; j < conf.dimy; j++) {
                    if (conf.site[i][j] == 0) {
                        g.setColor(pink);
                    } else {
                        g.setColor(blue);
                    }
                    g.fillOval((int) (i * dumx + dumx / 2) - 3, (int) (j * dumy + dumy / 2) - 3, 7, 7);
                }
            }
        }
        g.setColor(Color.black);
        g.fillOval((int) (conf.vacancyx * dumx + dumx / 2)-3, (int) (conf.vacancyy * dumy + dumy / 2)-3, 7, 7);
        if (conf.site[conf.newx][conf.newy] == 0) {
            g.setColor(pink);
        } else {
            g.setColor(blue);
        }
        g.fillOval((int) (conf.newx * dumx + dumx / 2)-3, (int) (conf.newy * dumy + dumy / 2)-3, 7, 7);
    }

    @Override
    public void update(Graphics g) {        
        paint(g);
    }
}